@@ -33,29 +33,42 @@ from shutil import copy2
 import cv2
 
 from .Preprocessing import PreprocessingMethods as PP
-#from .Barcodes import BarcodeMethods as BM
 
 # -- Functions for Segmentation
 def CalculateBackground(tiles):
+    '''
+    input: list of color or grayscale images each containing a single patch
+    output: scalar estimate of the intensity of the background agar based on the top 1/10th of each image
+    '''
     if len(np.shape(tiles[0]))==3:
         gray_tiles = [skimage.color.rgb2gray(t) for t in tiles]
     else:
         gray_tiles = tiles.copy()
-    
     # Compute average background value of agar
-    background_list = [np.median(t[0:20,0:-5]) for t in gray_tiles]
+    background_list = [np.median(t[0:int(np.shape(t)[0]/10),0:-5]) for t in gray_tiles]
     background = np.mean(background_list)
     return background
 
 def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
+    '''
+    input: required - color or grayscale tile containing a single patch
+                    - scalar estimate of background intensity of agar
+           optional
+                    - radius of the initial pin (minimum colony size) in number of pixels. Default value is 25. Use pin_size = 0 to exclude circle detection from the process.
+                    - method for intensity thresholding. Default value is 'otsu'. Currently this is the only option
+    output:
+            mask with 1's indicating a pixel is part of the colony and 0's indicating a pixel is part of the background
+    '''
     # -- convert to grayscale
     if len(np.shape(tile))==3:
         t = skimage.color.rgb2gray(tile)
     else:
         t = tile.copy()
     # -- initial intensity threshold    
+    ss = np.shape(t)[0]
     if threshold_method == 'otsu' or threshold_method == 'Otsu':
-        thresh = threshold_otsu(t[25:-25,25:-25])     
+        buffer = int(ss/8)
+        thresh = threshold_otsu(t[buffer:-buffer,buffer:-buffer])     
     else:
         print('Threshold method not recognized')
     # -- binarize and fill
@@ -65,7 +78,7 @@ def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
     label_objects, nb_labels = ndi.label(label_image>0)
     # Select largest object
     sizes = np.bincount(label_objects.ravel())
-    mask_sizes = sizes > 2000
+    mask_sizes = sizes > .05*(ss**2)
     mask_sizes[0] = 0
     colony_mask_temp = mask_sizes[label_objects]
     # remove artifacts connected to image border
@@ -75,8 +88,9 @@ def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
     # -- For small patches fit circle to tile with min radius of pin_size, max of pin_size + 10
     if pin_size > 0 and (np.sum(colony_mask) <= np.pi*(pin_size+10)**2 or thresh < background + 0.035): 
         # Circular Hough Transform
-        edges = 1*(t>np.mean(t[35:-35,35:-35])+.01)
-        edges_center = edges[35:-35,35:-35]
+        buffer = int(0.175*ss)
+        edges = 1*(t>np.mean(t[buffer:-buffer,buffer:-buffer])+.01)
+        edges_center = edges[buffer:-buffer,buffer:-buffer]
         hough_radii = np.arange(pin_size,pin_size+10, 1)
         hough_res = hough_circle(edges_center, hough_radii)
         centers = [None]*len(hough_radii) 
@@ -84,7 +98,7 @@ def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
         radii = [None]*len(hough_radii)
         for i, (radius, h) in enumerate(zip(hough_radii, hough_res)):
             # For each radius, extract one circle
-            peaks = peak_local_max(h, num_peaks=1, min_distance=25)
+            peaks = peak_local_max(h, num_peaks=1, min_distance=int(ss/8))
             centers[i] = peaks.tolist()[0]
             accums[i] = h[peaks[:, 0],peaks[:,1]].tolist()[0]
             radii[i] = radius
@@ -104,7 +118,7 @@ def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
             colony_mask = np.zeros_like(edges)
             try:
                 colony_mask_center[cx,cy] = 1
-                colony_mask[35:-35,35:-35] = colony_mask_center
+                colony_mask[buffer:-buffer,buffer:-buffer] = colony_mask_center
             except:
                 pass
             # Boolean-ize it
@@ -114,7 +128,7 @@ def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
                 label_objects, nb_labels = ndi.label(1*((label_image+colony_mask)>0))
                 # Select largest object
                 sizes = np.bincount(label_objects.ravel())
-                mask_sizes = sizes > 2000
+                mask_sizes = sizes > .05*(ss**2)
                 mask_sizes[0] = 0
                 colony_mask_temp = mask_sizes[label_objects]
                 # remove artifacts connected to image border
@@ -122,6 +136,12 @@ def SegmentTile(tile, background, pin_size=25, threshold_method='otsu'):
     return colony_mask
 
 def BuildSegmentedImage(image,colony_masks,corners):
+    '''
+    input: - original image that was segmented
+           - list of masks for each tile from the image
+           - list of top left corners indicating location of tile in the original image
+    return: black and white version of image with segmented colonies colored in    
+    '''
     if len(np.shape(image))==3:
         img = skimage.color.rgb2gray(image)
     else:
@@ -134,12 +154,15 @@ def BuildSegmentedImage(image,colony_masks,corners):
 # -- Functions for Quantitative Feature Extraction
 
 def SizeFeatures(tile,mask):
+    '''
+    input: tile containing a single colony of interest and corresponding mask indicating segmentation
+    returns: list containing area of colony, average pixel intensity of colony, sum of pixel intensities in colony, background intensity, perimeter of colony
+    '''
     if len(np.shape(tile))==3:
         gray_tile = skimage.color.rgb2gray(tile)
     else:
         gray_tile = tile.copy()
     clean_tile = tile*mask
-
     area = np.sum(1*mask)
     pixel_sum = np.sum(clean_tile)
     if area > 0:
@@ -151,6 +174,10 @@ def SizeFeatures(tile,mask):
     return [area, avg_int, pixel_sum, background, perimeter]
 
 def TextureFeatures(tile,mask):
+    '''
+    input: tile containing a single colony of interest and corresponding mask indicating segmentation
+    returns: list containing variance of pixel intensities in colony, complexity score (sum of second derivative of colony divided by area), and fraction of colony pixels falling into each of 10 local binary patterns
+    '''
     if len(np.shape(tile))==3:
         gray_tile = skimage.color.rgb2gray(tile)
     else:
@@ -174,7 +201,7 @@ def TextureFeatures(tile,mask):
         if len(temp) == 10:
                 LBP_score = temp
         else:
-            LBP_score = np.array([np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan])
+            LBP_score = np.array(10*[np.nan])
         # sobel filter
         clean_tile = tile*mask
         mask_temp = clean_tile>0.2
@@ -186,14 +213,18 @@ def TextureFeatures(tile,mask):
         C_score = np.sum(filt2[mask])/area
     else:
         variance = np.nan
-        LBP_score = np.array([np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan])
+        LBP_score = np.array(10*[np.nan])
         C_score = np.nan
     return [variance, C_score]+LBP_score.tolist()
 
-def BuildDF(tiles,masks,features ='all',save=True):
+def BuildDF(tiles, masks, features ='size'):
+    '''
+    input: list of tiles to be included in data frame, list of masks segmenting the colony in those tiles, features = 'size' to include area, pixel sum, etc. or features = 'all' to include texture features as well.
+    output: pandas dataframe with each row corresponding to a tile and each column corresponding to a feature. First column is "position" on the plate.
+    '''
     
     positions = ([x+y for x in ['A','B','C','D','E','F','G','H'] 
-            for y in ['1','2','3','4','5','6','7','8','9','10','11','12']])    
+                      for y in ['1','2','3','4','5','6','7','8','9','10','11','12']])    
     data = [None]*len(tiles)
     if features == 'all':
         features = (['Position','Area','AvgInt','PixelSum', 'Background', 'Perimeter','Variance', 'Complexity',
@@ -218,9 +249,28 @@ def BuildDF(tiles,masks,features ='all',save=True):
     return df
 
 def ProcessImage(file, sourcefolder, outputfolder,
-                 crop_method = 'Auto', pin_size = 25, crop_param = None, s = 200, array_dimensions = (8,12),
-                 adjust = True, rotate = False, save = True, display = False, calibrate = False):
-    
+                 crop_method = 'Auto', crop_param = None, s = 200, array_dimensions = (8,12), adjust = True, rotate = False, 
+                 pin_size = 25, features = 'size', save = True, display = False, calibrate = False):
+    '''
+    input:  
+            REQUIRED
+            - file
+            - sourcefolder
+            - outputfolder
+            OPTIONAL
+            - crop_method
+            - pin_size
+            - crop_param
+            - s
+            - features
+            - array_dimensions
+            - adjust
+            - rotate
+            - save
+            - display
+            - calibrate
+    output: - dataframe, tiles, masks, corners
+    '''
     image = PP.LoadImage(file, sourcefolder, rotate = rotate)
     
     if crop_method == 'Auto':
@@ -261,7 +311,7 @@ def ProcessImage(file, sourcefolder, outputfolder,
     masks = [SegmentTile(tile, background, pin_size=pin_size, threshold_method='otsu') for tile in tiles]
     
     image_label_overlay = BuildSegmentedImage(image,masks,corners)
-    df = BuildDF(tiles, masks, features ='all',save=True)
+    df = BuildDF(tiles, masks, features = features)
     
     if save:
         filename = file.split('.')[0]+'.csv'
@@ -278,8 +328,27 @@ def ProcessImage(file, sourcefolder, outputfolder,
     return df, tiles, masks, corners
     
 def Process1Image(file, sourcefolder, outputfolder,
-                 corners, pin_size = 25, s = 200, array_dimensions = (8,12),
-                 adjust = True, rotate = False, save = True, display = False):
+                 corners, s = 200, array_dimensions = (8,12), adjust = True, rotate = False, 
+                 pin_size = 25, features = 'size', save = True, display = False):
+    '''
+        input: 
+            REQUIRED
+            - file
+            - sourcefolder
+            - outputfolder
+            - corners
+            OPTIONAL
+            - pin_size
+            - s
+            - features
+            - array_dimensions
+            - adjust
+            - rotate
+            - save
+            - display
+            - calibrate
+    output: none --saves dataframe and segmented image if save = True
+    '''
     # -- Load Image
     image = PP.LoadImage(file, sourcefolder, rotate = rotate)
     # -- Set Corners
@@ -297,25 +366,45 @@ def Process1Image(file, sourcefolder, outputfolder,
     # -- Merge Segmented Plate image
     image_label_overlay = BuildSegmentedImage(image,masks,corners)
     # -- Extract Features
-    df = BuildDF(tiles, masks, features ='all',save=True)
+    df = BuildDF(tiles, masks, features = features)
     # -- Save Results
     if save:
         filename = file.split('.')[0]+'.csv'
         (df.to_csv(os.path.join(outputfolder, filename), index=False))
-        img_seg = 255*(image_label_overlay[200:-200,200:-200,:]) 
+        img_seg = 255*(image_label_overlay[s:-s,s:-s,:]) 
         A = Image.fromarray(img_seg.astype('uint8'), 'RGB')
         A.save(os.path.join(outputfolder,file.split('.')[0]+'_seg.jpg'))
     # -- Display Segmented Image
     if display:
         fig, axs = plt.subplots(1,1,figsize=(12,16))
-        axs.imshow(image_label_overlay[200:-200,200:-200,:]);
+        axs.imshow(image_label_overlay[s:-s,s:-s,:]);
         axs.axis('off')
         plt.show();
     return None    
         
 def ProcessBatch(sourcefolder, outputfolder,   
-                crop_method = 'Auto', s = 200, pin_size = 25, array_dimensions = (8,12),
-                adjust = True, rotate = False, save = True, display = False):
+                crop_method = 'Auto', s = 200, array_dimensions = (8,12), adjust = True, rotate = False, crop_param = None,
+                calibrate = True, pin_size = 25, features = 'size', save = True, display = False):
+    '''
+    input:
+            REQUIRED
+            -sourcefolder
+            -outputfolder
+            OPTIONAL
+            -crop_method
+            -tile side length
+            -array dimensions
+            - auto adjust
+            - rotate
+            - crop_param
+            - calibrate
+            - pin size
+            - features to compute 
+            - save -- True or False
+            - display -- True or False
+    output: none
+    '''
+    
     # -- Set Up Folders
     wd = os.getcwd()
     if save:
@@ -328,10 +417,10 @@ def ProcessBatch(sourcefolder, outputfolder,
     # -- Calibrate using first image
     file = image_list[0]
     df, tiles, masks, corners = ProcessImage(file, sourcefolder, outputfolder,
-                                     crop_method = crop_method, crop_param = None, s = s, 
+                                     crop_method = crop_method, crop_param = crop_param, s = s, 
                                      array_dimensions = array_dimensions, pin_size = pin_size,
-                                     adjust = adjust, rotate = rotate, 
-                                     save = False, display = True, calibrate = True)
+                                     features = features, adjust = adjust, rotate = rotate, 
+                                     save = False, display = True, calibrate = calibrate)
     PP.DisplayTiles(tiles)
     user_input = input("Would you like to proceed? y/n")
         
@@ -344,6 +433,7 @@ def ProcessBatch(sourcefolder, outputfolder,
                 'Array dimensions: '+str(array_dimensions),
                 'Autoadjust: '+str(adjust), 
                 'Rotate images: '+str(adjust),
+                'Features: '+features,
                 'SEGMENTATION PARAMETERS',
                 'Minimum intensity: 0.035 + background',
                 'Minimum area: np.pi*(pin_size+10)**2',
@@ -368,7 +458,7 @@ def ProcessBatch(sourcefolder, outputfolder,
             try:
                 Process1Image(file, sourcefolder, outputfolder,
                      corners, s = s, array_dimensions = array_dimensions, pin_size = pin_size,
-                     adjust = adjust, rotate = rotate, save = True, display = False)
+                     features = features, adjust = adjust, rotate = rotate, save = True, display = False)
                 count = count+1
             except:
                 missed_images.append(file)
@@ -387,33 +477,52 @@ def ProcessBatch(sourcefolder, outputfolder,
     
     
 ## -- Functions for loading example images
-def Funnel1():
-    stream = pkg_resources.resource_stream(__name__, 'Data/Funnel-1.jpg')
+def Funnel():
+    '''
+    input: none
+    output: returns sample image from Funnel cross data set
+    '''
+    stream = pkg_resources.resource_stream(__name__, 'Data/Im1.jpg')
     return skimage.io.imread(stream)
-def Funnel2():
-    stream = pkg_resources.resource_stream(__name__, 'Data/Funnel-2.jpg')
+def PSAT1():
+    '''
+    input: none
+    output: returns sample image from PSAT1 data set
+    '''
+    stream = pkg_resources.resource_stream(__name__, 'Data/Im2.jpg')
     return skimage.io.imread(stream)
-def Funnel3():
-    stream = pkg_resources.resource_stream(__name__, 'Data/Funnel-3.jpg')
-    return skimage.io.imread(stream)
-def PSAT11():
-    stream = pkg_resources.resource_stream(__name__, 'Data/PSAT1-1.jpg')
-    return skimage.io.imread(stream)
-def PSAT12():
-    stream = pkg_resources.resource_stream(__name__, 'Data/PSAT1-2.jpg')
-    return skimage.io.imread(stream)
-def OTC1():
-    stream = pkg_resources.resource_stream(__name__, 'Data/OTC-1.jpg')
-    return skimage.io.imread(stream)
-def OTC2():
-    stream = pkg_resources.resource_stream(__name__, 'Data/OTC-2.jpg')
+def OTC():
+    '''
+    input: none
+    output: returns sample image from OTC data set
+    '''
+    stream = pkg_resources.resource_stream(__name__, 'Data/Im3.jpg')
     return skimage.io.imread(stream)
 
 ## -- Function for processing example images
 
 def ProcessImageTest(image, outputfolder,crop_method = 'Auto', crop_param = None, s = 200, pin_size = 25, array_dimensions = (8,12),
-                 adjust = True, rotate = False, save = True, display = False, calibrate = False):
-    
+                 features = 'all', adjust = True, rotate = False, save = True, display = False, calibrate = False):
+    '''
+    Function to illustrate output of ProcessImage with the test images included in the package
+    input: 
+            REQUIRED
+            -image (array)
+            -outputfolder 
+            OPTIONAL
+            -crop_method
+            -crop_param
+            -s
+            -pin_size
+            -array_dimensions
+            -features
+            -adjust
+            -rotate
+            -save
+            -display
+            -calibrate            
+    output: returns data frame, tiles, masks, corners
+    '''
     if crop_method == 'Auto':
         tiles, corners = PP.AutoCropTest(image, s = s, array_dimensions = array_dimensions, 
                                   adjust=adjust, display = display)    
@@ -447,7 +556,7 @@ def ProcessImageTest(image, outputfolder,crop_method = 'Auto', crop_param = None
     masks = [SegmentTile(tile, background, pin_size=pin_size, threshold_method='otsu') for tile in tiles]
     
     image_label_overlay = BuildSegmentedImage(image,masks,corners)
-    df = BuildDF(tiles, masks, features ='all',save=True)
+    df = BuildDF(tiles, masks, features =features,save=True)
     
     if save:
         filename = 'TestImage.csv'
@@ -466,8 +575,28 @@ def ProcessImageTest(image, outputfolder,crop_method = 'Auto', crop_param = None
 ## -- Function for parallel processing batches of images 
    
 def ParallelProcessBatch(sourcefolder, outputfolder, numThreads = 2,   
-                crop_method = 'Auto', s = 200, pin_size = 25, array_dimensions = (8,12),
-                adjust = True, rotate = False, save = True, display = False):
+                crop_method = 'Auto', crop_param = None, s = 200, array_dimensions = (8,12), adjust = True, rotate = False,
+                pin_size = 25, features = 'size', save = True, display = False, calibrate = True):
+    '''
+    Function to process a batch of images in parallel
+            REQUIRED
+            -sourcefolder
+            -outputfolder 
+            OPTIONAL
+            -numThreads
+            -crop_method
+            -crop_param
+            -s
+            -pin_size
+            -array_dimensions
+            -features
+            -adjust
+            -rotate
+            -save
+            -display
+            -calibrate            
+    output: none
+    '''
     
     # -- Define workhorse function with input arguments
     def workMethod(tno, workQueue):
@@ -479,7 +608,7 @@ def ParallelProcessBatch(sourcefolder, outputfolder, numThreads = 2,
             try:
                 Process1Image(str(item), sourcefolder, outputfolder,
                              corners, s = s, pin_size = pin_size, array_dimensions = array_dimensions,
-                             adjust = adjust, rotate = rotate, save = True, display = False)
+                             features = features, adjust = adjust, rotate = rotate, save = True, display = False)
                 count = count + 1
             except:
                 missed_images.append(file)
@@ -509,10 +638,10 @@ def ParallelProcessBatch(sourcefolder, outputfolder, numThreads = 2,
     # -- Calibrate using first image
     file = image_list[0]
     df, tiles, masks, corners = ProcessImage(file, sourcefolder, outputfolder,
-                                     crop_method = crop_method, crop_param = None, s = s, 
+                                     crop_method = crop_method, crop_param = crop_param, s = s, 
                                      pin_size = pin_size, array_dimensions = array_dimensions,
-                                     adjust = adjust, rotate = rotate, 
-                                     save = False, display = True, calibrate = True)
+                                     adjust = adjust, rotate = rotate, features = featues,
+                                     save = False, display = True, calibrate = calibrate)
     PP.DisplayTiles(tiles)
     user_input = input("Would you like to proceed? y/n")
         
@@ -525,6 +654,7 @@ def ParallelProcessBatch(sourcefolder, outputfolder, numThreads = 2,
                 'Array dimensions: '+str(array_dimensions),
                 'Autoadjust: '+str(adjust), 
                 'Rotate images: '+str(adjust),
+                'Features: ' +features,
                 'SEGMENTATION PARAMETERS',
                 'Minimum intensity: 0.035 + background',
                 'Minimum area: np.pi*(pin_size+10)**2',
